/***************************************************************************
 *   Copyright Brian Ledbetter 2001-2003 <brian@shadowcom.net>             *
 *   Copyright Ravikiran Rajagopal 2003                                    *
 *   ravi@ee.eng.ohio-state.edu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License (version 2) as   *
 *   published by the Free Software Foundation. (The original KSplash/ML   *
 *   codebase (upto version 0.95.3) is BSD-licensed.)                      *
 *                                                                         *
 ***************************************************************************/
/* $Id: icon.cpp,v 1.3 2003/03/28 11:01:23 mlaurent Exp $ */

#include <kcursor.h>
#include <kdebug.h>
#include <kstandarddirs.h>
#include <kwin.h>

#include <qapplication.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qwidget.h>

#include "icon.h"
#include "icon.moc"
/*
 * Icon::Icon(): Displays an icon on the screen.
 */
Icon::Icon(
  unsigned int icon_num,
  unsigned int icon_std_width,
  const KPixmap& pix,
  const QString& text,
  Position icon_position,
  bool iconsJumping
)
    :QHBox( 0, "wndSplash", WStyle_Customize|WStyle_StaysOnTop|WStyle_NoBorder )
{

  mIconPos = icon_position;
  mIconNum = icon_num;
  mIconSize = icon_std_width;
  mIconJumping = iconsJumping;

  KWin::setState( winId(), NET::Modal | NET::StaysOnTop | NET::SkipTaskbar | NET::SkipPager );
  KWin::setType( winId(), NET::Dock );
  setFrameStyle( QFrame::NoFrame );

  QLabel *w = new QLabel( this );
  w->setFixedSize( pix.width(), pix.height() );
  w->setPixmap( pix );

  /*
  * Note: This mode is imprecise, and doesn't correctly handle icon alpha channels. It
  * would be preferrable to draw this with NoBackground, but this feature is only present
  * in Qt3. Furthermore, if the background were to change, we would not be redrawn again,
  * and the icon would be left looking out of place. I will leave this code in place
  * until I can figure out a better way to force a background re-render in Qt (if possible).
  */
  if(pix.mask())
  {
    setMask(*pix.mask());
    w->setMask(*pix.mask());
  }

  setBackgroundMode( NoBackground );
  w->setBackgroundMode( NoBackground );

  mVelocity = mInitialVelocity = 8.0;
  mGravity = 0.8;

  resize( pix.width(), pix.height() );
  mStatusText = text;

  QPoint p = determinePosition();
  mGroundX = mPosX = p.x();
  mGroundY = mPosY = p.y();
  move( p );

  mStopJump = false;
  if( mIconJumping )
  {
    QTimer *t = new QTimer( this );
    connect(t, SIGNAL(timeout()), SLOT(slotJump()));
    t->start( 50, false );
  }
}

void Icon::show()
{
  emit setStatusText( mStatusText );
  QHBox::show();
}

void Icon::noshow()
{
  emit setStatusText( mStatusText );
}

/*
 * Icon::determinePosition(): Based on the information we've
 * stored within us, determine where we should be placed on the
 * screen. This is placed in its own function due to the massive
 * complexity required by the variables involved.
 */
QPoint Icon::determinePosition()
{
  int DW, DH, SBH, wid, X, Y, x, y, nSlot, topshift, bottomshift;

  bottomshift = topshift = 0;
  // Ravi - check Xinerama correctness later.
  DW = QApplication::desktop()->width();
  DH = QApplication::desktop()->height();
  SBH = 0;
  wid = mIconSize;
  x = mIconNum;
  y = 1;

  topshift = bottomshift = 0;

  // Different starting positions require different positioning
  // rules. The horizontal rules and the vertical rules can be
  // grouped together, as they are innately similar.
  switch( mIconPos )
  {
    // HORIZONTAL: Top Left -> Top Right
  case( Icon::HTopLeft ):
    nSlot = ( DW / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (3) + ((x - 1) * wid);
    Y = topshift + 3 + ( (y-1) * wid );
    break;

    // HORIZONTAL: Bottom Left -> Bottom Right
  case( Icon::HBottomLeft ):
    nSlot = ( DW / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (3) + ((x - 1) * wid);
    Y = (DH-3) - (y * wid) - bottomshift;
    break;

    // HORIZONTAL: Top Right -> Top Left
  case( Icon::HTopRight ):
    nSlot = ( DW / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (DW - 3) - (( x ) * wid );
    Y = topshift + (3) + ( (y-1) * wid );
    break;

    // HORIZONTAL: Bottom Right -> Bottom Left
  case( Icon::HBottomRight ):
    nSlot = ( DW / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (DW - 3) - (( x ) * wid );
    Y = (DH-3) - (y * wid) - bottomshift;
    break;

    // VERTICAL: Top Left -> Bottom Left
  case( Icon::VTopLeft ):
    nSlot = ( DH / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (3) + (( y - 1 ) * wid );
    Y = topshift + (3) + ((x-1) * wid);
    break;

    // VERTICAL: Top Right -> Bottom Right
  case( Icon::VTopRight ):
    nSlot = ( DH / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (DW - 3) - (( y ) * wid );
    Y = topshift + (3) + ((x-1) * wid);
    break;

    // VERTICAL: Bottom Left -> Top Left
  case( Icon::VBottomLeft ):
    nSlot = ( DH / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (3) + (( y - 1 ) * wid );
    Y = (DH-3) - (x * wid) - bottomshift;
    break;

    // VERTICAL: Bottom Right -> Top Right
  case( Icon::VBottomRight ):
    nSlot = ( DH / wid );
    while( x > nSlot )
    {
      x = QMAX(x,x-nSlot);
      y++;
    }
    X = (DW - 3) - (( y ) * wid );
    Y = (DH-3) - (x * wid) - bottomshift;
    break;

    // Unknown Condition. Reconfigure variable and
    // retry. Recursively, of course! ;)
  default:
    mIconPos = HBottomLeft;
    return(determinePosition());
    break;
  }
  return( QPoint( X, Y ) );
}

// Make the icons jump in a macosx-like way (Thanks, Jone!:)
void Icon::slotJump()
{
  // Based on our position, bounce around.
  switch( mIconPos )
  {
    // LEFT
  case( Icon::VTopLeft ):
  case( Icon::VBottomLeft ):
    mVelocity -= mGravity;
    if( mStopJump )
    {
      if( mPosX <= mGroundX )
      {
        mPosX = mGroundX;
        mVelocity = mGravity = 0.0;
      }
    }
    else if( mPosX <= mGroundX ) mVelocity = mInitialVelocity;
    mPosX = (int)((float)mPosX + mVelocity);
    break;

    // RIGHT
  case( Icon::VTopRight ):
  case( Icon::VBottomRight ):
    mVelocity -= mGravity;
    if( mStopJump )
    {
      if( mPosX >= mGroundX )
      {
        mPosX = mGroundX;
        mVelocity = mGravity = 0.0;
      }
    }
    else if( mPosX >= mGroundX ) mVelocity = mInitialVelocity;
    mPosX = (int)((float)mPosX - mVelocity);
    break;

    // TOP
  case( Icon::HTopLeft ):
  case( Icon::HTopRight ):
    mVelocity -= mGravity;
    if( mStopJump )
    {
      if( mPosY <= mGroundY )
      {
        mPosY = mGroundY;
        mVelocity = mGravity = 0.0;
      }
    }
    else if( mPosY <= mGroundY ) mVelocity = mInitialVelocity;
    mPosY = (int)((float)mPosY + mVelocity );
    break;

    // BOTTOM
  case( Icon::HBottomLeft ):
  case( Icon::HBottomRight ):
  default:
    mVelocity -= mGravity;
    if( mStopJump )
    {
      if( mPosY >= mGroundY )
      {
        mPosY = mGroundY;
        mVelocity = mGravity = 0.0;
      }
    }
    else if( mPosY >= mGroundY ) mVelocity = mInitialVelocity;
    mPosY = (int)((float)mPosY - mVelocity );
    break;
  }
  move( QPoint( mPosX, mPosY ) );
}

void Icon::slotStopJumping()
{
  mStopJump = true;
}
